#  Copyright (C) 2016-2019  Kyle.Hwang <upday7[at]163.com>
#  #
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version, with the additions
#  listed at the end of the accompanied license file.
#  #
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#  #
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
#  #
#  NOTE: This program is subject to certain additional terms pursuant to
#  Section 7 of the GNU Affero General Public License.  You should have
#  received a copy of these additional terms immediately following the
#  terms and conditions of the GNU Affero General Public License which
#  accompanied this program.

import re

from bs4 import Tag

from .base_provider import BaseProvider
from ..parser import Parser


class _ExampleProvider(Parser):
    def val_sent(self):
        sent = self.select("em", one=False)[0]
        if not sent:
            sent = self.bs.contents
        return re.search("‘?(?P<c>.+)’?", sent).group("c") if sent else None

    def val_trans(self):
        try:
            return self.select("em", one=False)[1]
        except IndexError:
            return None


class _ExpEntryProvider(Parser):
    @property
    def val_indicators(self):
        return self.select("span.indicators")

    @property
    def val_trans(self):
        trans = self.select("div.tr")
        return re.search("‘?(?P<c>.+)’?", trans).group("c") if trans else None

    @property
    def val_exp(self):
        exp = self.select("div.exg.em")
        if not exp:
            exp = self.select(".ind")
        return re.search("‘?(?P<c>.+)’?", exp).group("c") if exp else None

    @property
    def val_examples(self):
        _ = self.provider_to_list(
            _ExampleProvider, "div.examples > div.exg > ul > li.ex"
        )
        _.insert(
            0, _ExampleProvider(self.select(".ex", one=True, text=False)).to_dict()
        )
        return _


class _GrambProvider(Parser):
    @property
    def val_pos(self):
        return self.select("span.pos")

    @property
    def val_exps(self):
        return self.provider_to_list(_ExpEntryProvider, "ul.semb > li")


class _PrimaryHeaderProvider(Parser):
    def val_head_word(self):
        sup = self.select("span.hw > sup")
        return self.select("span.hw").replace(sup if sup else "", "")

    def val_trans(self):
        return self.select("span.head-translation")


class _PhraseProvider(Parser):  # class_=senseInnerWrapper
    def val_phrases(self):
        return self.select("strong.phrase", one=False)

    def val_exps(self):
        return self.provider_to_list(_ExpEntryProvider, "ul.semb > li")


class Lexico(BaseProvider):
    @property
    def url(self):
        _ = "definition"
        if self.seg in ("es-en",):
            _ = "traducir"
        if self.seg in ("en-es",):
            _ = "translate"
        if self.seg in ("es",):
            _ = "definicion"
        return f"https://www.lexico.com/{self.seg}/{_}/{self.word}"

    def __init__(self, word: str, seg="es-en"):
        super(Lexico, self).__init__(word, seg)
        self.seg = seg

    @property
    def head_word(self):
        return _PrimaryHeaderProvider(
            self.select(".primary_homograph", text=False)
        ).to_dict()

    @property
    def val_audio(self):
        return self.select("a.headwordAudio.rsbtn_play", text=False).audio["src"]

    @property
    def val_defs(self):
        return self.provider_to_list(_GrambProvider, "section.gramb")

    @property
    def val_phrases(self):
        phrase_title_tag: Tag = self.bs.find("h3", class_="phrases-title")
        if not phrase_title_tag:
            return []
        return self.provider_to_list(
            _PhraseProvider, "section.etymology.etym > div.senseInnerWrapper"
        )



#  Copyright (C) 2016-2019  Kyle.Hwang <upday7[at]163.com>
#  #
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version, with the additions
#  listed at the end of the accompanied license file.
#  #
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#  #
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
#  #
#  NOTE: This program is subject to certain additional terms pursuant to
#  Section 7 of the GNU Affero General Public License.  You should have
#  received a copy of these additional terms immediately following the
#  terms and conditions of the GNU Affero General Public License which
#  accompanied this program.

import json
from urllib.parse import quote as urlquote
from urllib.request import urlopen

UD_DEFID_URL = "https://api.urbandictionary.com/v0/define?defid="
UD_DEFINE_URL = "https://api.urbandictionary.com/v0/define?term="
UD_RANDOM_URL = "https://api.urbandictionary.com/v0/random"


class UrbanDefinition(object):
    def __init__(self, word, definition, example, upvotes, downvotes):
        self.word = word
        self.definition = definition
        self.example = example
        self.upvotes = upvotes
        self.downvotes = downvotes

    def __str__(self):
        return "%s: %s%s (%d, %d)" % (
            self.word,
            self.definition[:50],
            "..." if len(self.definition) > 50 else "",
            self.upvotes,
            self.downvotes,
        )


def _get_urban_json(url):
    f = urlopen(url)
    data = json.loads(f.read().decode("utf-8"))
    f.close()
    return data


def _parse_urban_json(json, check_result=True):
    result = []
    if json is None or any(e in json for e in ("error", "errors")):
        raise Exception("UD: Invalid input for Urban Dictionary API")
    if check_result and ("list" not in json or len(json["list"]) == 0):
        return result
    for definition in json["list"]:
        d = UrbanDefinition(
            definition["word"],
            definition["definition"],
            definition["example"],
            int(definition["thumbs_up"]),
            int(definition["thumbs_down"]),
        )
        result.append(d)
    return result


def define(term):
    """Search for term/phrase and return list of UrbanDefinition objects.

    Keyword arguments:
    term -- term or phrase to search for (str)
    """
    json = _get_urban_json(UD_DEFINE_URL + urlquote(term))
    return _parse_urban_json(json)


def defineID(defid):
    """Search for UD's definition ID and return list of UrbanDefinition objects.

    Keyword arguments:
    defid -- definition ID to search for (int or str)
    """
    json = _get_urban_json(UD_DEFID_URL + urlquote(str(defid)))
    return _parse_urban_json(json)


def random():
    """Return random definitions as a list of UrbanDefinition objects."""
    json = _get_urban_json(UD_RANDOM_URL)
    return _parse_urban_json(json, check_result=False)



#  Copyright (C) 2016-2019  Kyle.Hwang <upday7[at]163.com>
#  #
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version, with the additions
#  listed at the end of the accompanied license file.
#  #
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#  #
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
#  #
#  NOTE: This program is subject to certain additional terms pursuant to
#  Section 7 of the GNU Affero General Public License.  You should have
#  received a copy of these additional terms immediately following the
#  terms and conditions of the GNU Affero General Public License which
#  accompanied this program.


from urllib.parse import parse_qs

from .base_provider import BaseProvider
from ..parser import Parser


class _SuggestionProvider(Parser):
    def val_phrase(self):
        return parse_qs(self.bs["url"].split("?")[-1])["q"][0].replace("+", " ")

    def val_exp(self):
        return self.bs["query"].replace(self.val_phrase(), "").strip()


class CNBingSuggestion(BaseProvider):
    @property
    def url(self):
        return (
            f"https://cn.bing.com/AS/Suggestions?scope=dictionary&pt=page.bingdict"
            f"&bq=dict&mkt=zh-cn&ds=bingdict&qry={self.word}"
            f"&cp=6&cvid=DCBD6682795F4F4CBE6CA0809F43ED3C"
        )

    def __init__(self, word: str, seg=""):
        super(CNBingSuggestion, self).__init__(word, seg)
        self.seg = seg

    @property
    def head_word(self):
        return None

    @property
    def val_suggestion(self):
        return self.provider_to_list(_SuggestionProvider, "li.sa_sg")



#  Copyright (C) 2016-2019  Kyle.Hwang <upday7[at]163.com>
#  #
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version, with the additions
#  listed at the end of the accompanied license file.
#  #
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#  #
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
#  #
#  NOTE: This program is subject to certain additional terms pursuant to
#  Section 7 of the GNU Affero General Public License.  You should have
#  received a copy of these additional terms immediately following the
#  terms and conditions of the GNU Affero General Public License which
#  accompanied this program.

import abc
import re
import unicodedata
from typing import Union, Tuple

import requests
from bs4 import BeautifulSoup, Tag
from requests import Response, HTTPError

from .utils import STD_HEADERS


def _norm_str(s: str):
    return re.sub(r"\s+", " ", unicodedata.normalize("NFKD", s)).strip()


class Parser:
    to_dict_fields = ()

    def __init__(self, markup: Union[BeautifulSoup, str, Tag]):
        self._markup = markup
        self._bs = None

    @property
    def markup(self) -> Union[BeautifulSoup, str, Tag]:
        return self._markup

    @property
    def bs(self) -> Union[BeautifulSoup, Tag]:
        if not self._bs:
            if isinstance(self.markup, (BeautifulSoup, Tag)):
                self._bs = self.markup
            else:
                self._bs = BeautifulSoup(markup=self.markup, features="html.parser")
        return self._bs

    def select(self, p: str, one=True, text=True):
        if one:
            t = self.bs.select_one(p)
            if t and text:
                return _norm_str(t.text)
            return t
        else:
            ts = self.bs.select(p)
            if ts and text:
                return [_norm_str(t.text) for t in ts]
            return ts

    def get_by_cls(self, name: str, cls: str, **attrs) -> Tag:
        return self.bs.find(name, class_=cls, attrs=attrs)

    def to_dict(self):
        _ = {}

        fields = []
        fields.extend(self.to_dict_fields)
        fields.extend(
            [
                prop.lower().split("val_")[-1]
                for prop in dir(self)
                if prop.startswith("val_")
            ]
        )

        for field in set([f.lower() for f in fields]):
            try:
                val = getattr(self, field, getattr(self, f"val_{field}", None))
                if callable(val):
                    val = val()
                if isinstance(val, str):
                    val = unicodedata.normalize("NFKD", val)
                    # remove brackets
                    m = re.match(r"\((?P<c>.+)\)", val)
                    if m:
                        val = m.group("c")
                if val not in [None, ""]:
                    _[field.split("val_")[-1]] = val
            except AttributeError:
                ...
                # _[field.split('val_')[-1]] = None
        return _

    def provider_to_list(
        self,
        provider_cls,
        block_selector: Union[str, Tuple[str, dict],],
        find_in_tag: Union[BeautifulSoup, Tag] = None,
    ):
        if not find_in_tag:
            find_in_tag = self.bs
        try:
            if isinstance(block_selector, str):
                blocks = self.select(block_selector, one=False, text=False)
            else:
                blocks = find_in_tag.find_all(block_selector[0], **block_selector[1])
        except AttributeError:
            return []
        return [provider_cls(d).to_dict() for d in blocks]


class WebParser(Parser):
    def __init__(self, word: str):
        self.word = word
        super(WebParser, self).__init__(markup="")
        self._rsp = None

    @property
    @abc.abstractmethod
    def url(self):
        ...

    @property
    def rsp(self) -> Response:
        if not self._rsp:
            self._rsp = requests.get(self.url, headers=STD_HEADERS)
            self._rsp.raise_for_status()
        return self._rsp

    @property
    def json(self) -> dict:
        return self.rsp.json()

    @property
    def markup(self) -> str:
        if not self._markup:
            try:
                self._markup = self.rsp.content.decode()
            except HTTPError:
                self._markup = ""
        return self._markup



#  Copyright (C) 2016-2019  Kyle.Hwang <upday7[at]163.com>
#  #
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version, with the additions
#  listed at the end of the accompanied license file.
#  #
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#  #
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
#  #
#  NOTE: This program is subject to certain additional terms pursuant to
#  Section 7 of the GNU Affero General Public License.  You should have
#  received a copy of these additional terms immediately following the
#  terms and conditions of the GNU Affero General Public License which
#  accompanied this program.

import json
import unittest

from web_dict.core.factory import VocabularySuggestionDictionary


class VocabularyTest(unittest.TestCase):
    def setUp(self) -> None:
        # self.c = VocabularyDictionary(word='python')
        self.c = VocabularySuggestionDictionary(word="python")

    def test_basic(self):
        self.assertIsNotNone(self.c.do_search(),)
        self._p(self.c.do_search())

    def _p(self, c):
        print(json.dumps(c, indent=4, ensure_ascii=False))


if __name__ == "__main__":
    unittest.main()



#  Copyright (C) 2016-2019  Kyle.Hwang <upday7[at]163.com>
#  #
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version, with the additions
#  listed at the end of the accompanied license file.
#  #
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#  #
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
#  #
#  NOTE: This program is subject to certain additional terms pursuant to
#  Section 7 of the GNU Affero General Public License.  You should have
#  received a copy of these additional terms immediately following the
#  terms and conditions of the GNU Affero General Public License which
#  accompanied this program.

import re

from .base_provider import BaseProvider
from ..parser import Parser


class _BriefDefProvider(Parser):
    def val_pos(self):
        return self.select("span.pos")

    def val_def(self):
        return self.select("span.def > span")


class _AuthDefSegProvider(Parser):
    def val_topic(self):
        return [
            {"en": ent.text, "zh": zht.text}
            for ent, zht in zip(
                self.select("span.val_dis", one=False, text=False),
                self.select("span.bil_dis", one=False, text=False),
            )
        ]

    def val_defs(self):
        return [
            {"en": ent.text, "zh": zht.text}
            for ent, zht in zip(
                self.select("div.def_pa", text=False).find_all("span", class_="val"),
                self.select("div.def_pa", text=False).find_all("span", class_="bil"),
            )
        ]

    def val_examples(self):
        return [
            {"en": ent.text, "zh": zht.text}
            for ent, zht in zip(
                self.select("div.li_exs", text=False).find_all("div", class_="val_ex"),
                self.select("div.li_exs", text=False).find_all("div", class_="bil_ex"),
            )
        ]


class _AuthDefProvider(Parser):
    def val_pos(self):
        return self.select("div.pos",)

    def val_def(self):
        return self.provider_to_list(_AuthDefSegProvider, "div.de_seg")

    def val_idm(self):
        idm_tag = self.bs.find("div", class_="idm_seg")
        if not idm_tag:
            return []
        try:
            return [
                {"idm": idm.text, "example": {"en": ent.text, "zh": zht.text}}
                for idm, ent, zht in zip(
                    idm_tag.find_all("span", class_="ids"),
                    idm_tag.find_all("span", class_="val"),
                    idm_tag.find_all("span", class_="bil"),
                )
            ]
        except AttributeError:
            return []


class _EnZhDefProvider(Parser):
    def val_pos(self):
        return self.select("div.pos.pos1")

    def val_defs(self):
        return list(set(self.select("div.df_cr_w", one=False)))


class _EnEnDefProvider(Parser):
    def val_pos(self):
        return self.select("div.pos.pos1")

    def val_defs(self):
        return list(set(self.select("div.df_cr_w", one=False)))


class _DefsProvider(Parser):
    def val_auth(self):
        return self.provider_to_list(_AuthDefProvider, "div.each_seg")

    def val_ec(self):
        return self.provider_to_list(_EnZhDefProvider, "div#crossid > table > tr")

    def val_ee(self):
        return self.provider_to_list(_EnEnDefProvider, "div#homoid > table > tr")


class CNBing(BaseProvider):
    @property
    def url(self):
        return f"https://cn.bing.com/dict/search?q={self.word}"

    def __init__(self, word: str, seg=""):
        super(CNBing, self).__init__(word, seg)
        self.seg = seg

    @property
    def head_word(self):
        return self.select("div.hd_div > h1 > strong")

    @property
    def val_phone(self):
        p = re.compile(r"\[(?P<c>.+)\]")
        m = re.search(p, self.select("div.hd_pr")) if self.select("div.hd_pr") else None
        m2 = (
            re.search(p, self.select("div.hd_prUS"))
            if self.select("div.hd_prUS")
            else None
        )
        return {"uk": m.group("c") if m else None, "us": m2.group("c") if m2 else None}

    @property
    def val_images(self):
        return [
            t["src"].replace("&w=80&h=80", "")
            for t in self.select(
                "div.img_area > div.simg > a > img", one=False, text=False
            )
        ]

    @property
    def val_brief(self):
        return self.provider_to_list(
            _BriefDefProvider,
            ("li", {}),
            find_in_tag=self.bs.find("div", class_="qdef"),
        )

    @property
    def val_defs(self):
        return self.provider_to_list(_DefsProvider, "div#defid")



#  Copyright (C) 2016-2019  Kyle.Hwang <upday7[at]163.com>
#  #
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version, with the additions
#  listed at the end of the accompanied license file.
#  #
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#  #
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
#  #
#  NOTE: This program is subject to certain additional terms pursuant to
#  Section 7 of the GNU Affero General Public License.  You should have
#  received a copy of these additional terms immediately following the
#  terms and conditions of the GNU Affero General Public License which
#  accompanied this program.

import re

from .base_provider import BaseProvider
from ..parser import Parser


class _PhraseProvider(Parser):
    to_dict_fields = (
        "phrase",
        "trans",
    )

    @property
    def phrase(self):
        return self.select("span.type-phr > span.orth")

    @property
    def trans(self):
        return self.select("span.cit.type-translation > span.quote")


class _IdiomProvider(Parser):
    to_dict_fields = ("orth", "trans")

    @property
    def orth(self):
        return self.select("span.orth")

    @property
    def trans(self):
        return self.select("span.cit.type-translation > span")


class _GrammarProvider(Parser):
    to_dict_fields = {
        "content",
        "trans",
    }

    @property
    def content(self):
        content = self.select("span.colloc")
        if content:
            return re.match(r"\[(?P<c>.+)\]", content).group("c")
        return content

    @property
    def trans(self):
        return self.select("span.cit.type-translation")


class _ExampleProvider(Parser):
    to_dict_fields = {"sent", "trans"}

    @property
    def sent(self):
        try:
            return re.sub(r"\s+", " ", self.contents[0]).strip()
        except IndexError:
            return None

    @property
    def trans(self):
        try:
            return re.sub(r"\s+", " ", "; ".join(self.contents[1:])).strip()
        except IndexError:
            return None

    @property
    def contents(self):
        _ = self.select("span.quote", one=False)
        if len(_) == 2:
            return _
        return [t.text.replace("⇒", "").strip() for t in self.bs.find_all("q")]


class _SenseProvider(Parser):
    to_dict_fields = (
        "exp",
        "examples",
        "syn",
        "grammars",
        "idioms",
        "phrases" "senses",
    )

    @property
    def syn(self):
        syn_pattern = re.compile(r"\)?\s*\(=?\s*(?P<c>.+)\)")
        t1 = self.select("span.lbl.type-geo > span.lbl.type-syn", text=False)
        if t1:
            syn = re.match(syn_pattern, t1.text).group("c")
            geo_txt = self.select("span.lbl.type-geo")

            return {
                "syn": syn,
                "geo": re.match(r"\((?P<g>.+)\)\s*\(.+\)", geo_txt).group("g"),
            }
        try:
            syn = re.match(syn_pattern, self.select("span.lbl.type-syn")).group("c")
        except:
            syn = ""
        return {"syn": syn, "geo": None}

    @property
    def exp(self):
        exp = "; ".join(
            t.text
            for t in self.bs.find_all(
                "span",
                class_=re.compile(r"cit\s(cit-)?type-translation"),
                recursive=False,
            )
        )

        if not exp:
            exp = self.select("div.def")
        return exp

    @property
    def idioms(self):
        return self.provider_to_list(_IdiomProvider, "div.re.type-idm")

    @property
    def examples(self):
        return self.provider_to_list(
            _ExampleProvider, "div.cit.type-example"
        ) or self.provider_to_list(_ExampleProvider, "div.cit.cit-type-example")

    @property
    def phrases(self):
        return self.provider_to_list(_PhraseProvider, "div.type-phr")

    # @property
    # def grammars(self):
    #     return self.provider_to_list(_GrammarProvider, "div.sense")

    def val_senses(self):
        return self.provider_to_list(_SenseProvider, "div.sense")


class _DefProvider(Parser):
    to_dict_fields = ("pos", "senses", "misc")

    def __init__(self, markup: str):
        super(_DefProvider, self).__init__(markup=markup)

    @property
    def pos(self):
        return self.select("span.pos")

    @property
    def misc(self):
        misc = self.select("span.lbl.type-misc")
        return re.match(r"\((?P<c>.+)\)", misc).group("c") if misc else misc

    @property
    def senses(self):
        senses = [
            s
            for s in self.provider_to_list(
                _SenseProvider, ("div", dict(class_="sense", recursive=False))
            )
            if any(s.values())
        ]
        if not senses:
            senses = [
                s
                for s in self.provider_to_list(
                    _SenseProvider,
                    ("li", dict(class_="sense_list_item", recursive=True)),
                )
                if any(s.values())
            ]
        return senses


class CollinsWeb(BaseProvider):
    to_dict_fields = ("head_word", "pron", "rank", "audio", "defs")

    @property
    def url(self):
        url = f"https://www.collinsdictionary.com/dictionary/{self.seg}/{self.word}"
        print(f"Requesting {url}")
        return url

    def __init__(self, word: str, seg: str = "spanish-english"):
        super(CollinsWeb, self).__init__(word, seg)

    @property
    def bs(self):
        bs = super(BaseProvider, self).bs
        return bs.find("div", class_=re.compile(r"cB\scB-def.+")) or bs.find(
            "div", class_=re.compile(r"cB\scB-t")
        )

    @property
    def pron(self):
        try:
            return self.select("span.pron.type-") or re.match(
                r"\((?P<pron>.+)\)", self.select("div.cB-h > div > span.pron")
            ).group("pron")
        except TypeError:
            return None

    @property
    def rank(self):
        try:
            return int(
                self.select("span.word-frequency-img", one=True, text=False)[
                    "data-band"
                ]
            )
        except Exception as exc:
            return None

    @property
    def head_word(self):
        try:
            return self.select("h2.h2_entry > span.orth") or self.select("h2.h2_entry")
        except AttributeError:
            return None

    @property
    def audio(self):
        return self.select("a.audio_play_button", text=False).get("data-src-mp3")

    @property
    def defs(self):
        return [
            s for s in self.provider_to_list(_DefProvider, "div.hom") if any(s.values())
        ]



#  Copyright (C) 2016-2019  Kyle.Hwang <upday7[at]163.com>
#  #
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version, with the additions
#  listed at the end of the accompanied license file.
#  #
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#  #
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
#  #
#  NOTE: This program is subject to certain additional terms pursuant to
#  Section 7 of the GNU Affero General Public License.  You should have
#  received a copy of these additional terms immediately following the
#  terms and conditions of the GNU Affero General Public License which
#  accompanied this program.

from distutils.core import setup
from pathlib import Path

from setuptools import find_packages

try:
    long_description = Path("readme.md").read_text()
except FileNotFoundError:
    long_description = ""

setup(
    name="web_dict",  # How you named your package folder (MyLib)
    packages=find_packages(),  # Chose the same as "name"
    description="parser class for vaiouse online-dict, e.g. collinsdictionary/lexico/vocabulary/spanishdict etc.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    version="0.1.84",  # Start with a small number and increase it with every change you make
    license="agpl-3.0",  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    author="Kyle, Hwang",  # Type in your name
    author_email="upday7@163.com",  # Type in your E-Mail
    url="https://github.com/upday7/web_dict",  # Provide either the link to your github or to your website
    keywords=[
        "dictionary",
        "spanish",
        "english",
        "chinese",
        "collins",
        "oxford",
        "lexico",
    ],
    install_requires=["bs4", "requests",],  # I get to this in a second
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable"
        "Intended Audience :: Developers",  # Define that your audience are developers
        "Programming Language :: Python :: 3.8",
    ],
)
